//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.


#include <vector>
#include <set>

#include "io/VtkWriter.h"
#include "parallel/PetscProblemStat.h"
#include "parallel/PetscSolver.h"
#include "parallel/MpiHelper.h"
#include "parallel/BddcMlSolver.h"

namespace AMDiS {

  using namespace std;


  PetscProblemStat::PetscProblemStat(string nameStr,
				     ProblemIterationInterface *problemIteration)
    : ParallelProblemStatBase(nameStr, problemIteration),
      deleteSolver(true)
  {
    FUNCNAME("PetscProblemStat::PetscProblemStat()");

    string tmp("");
    Parameters::get("parallel->solver", tmp);
    
    if (tmp == "petsc-schur") {
      petscSolver = new PetscSolverSchur();
    } else if (tmp == "petsc-feti") {
      petscSolver = new PetscSolverFeti();
    } else if (tmp == "petsc-block") {
      petscSolver = new PetscSolverGlobalBlockMatrix();
    } else if (tmp == "petsc" || tmp == "") {
      petscSolver = new PetscSolverGlobalMatrix();
    } else if (tmp == "bddcml") {
#ifdef HAVE_BDDC_ML
      petscSolver = new BddcMlSolver();
#else
      ERROR_EXIT("AMDiS was compiled without BDDC-ML support!\n");
#endif
    } else {
      ERROR_EXIT("No parallel solver %s available!\n", tmp.c_str());
    }

    tmp = "";
    Parameters::get(nameStr + "->solver->petsc prefix", tmp);
    petscSolver->setKspPrefix(tmp);    
  }


  PetscProblemStat::PetscProblemStat(string nameStr,
				     PetscSolver *solver)
    : ParallelProblemStatBase(nameStr, NULL),
      petscSolver(solver),
      deleteSolver(false)
  {}


  void PetscProblemStat::initialize(Flag initFlag, 
				    ProblemStatSeq* adoptProblem,
				    Flag adoptFlag)
  {
      ParallelProblemStatBase::initialize(initFlag, adoptProblem, adoptFlag);

      meshDistributor->setBoundaryDofRequirement(petscSolver->getBoundaryDofRequirement());
  }


  void PetscProblemStat::solve(AdaptInfo *adaptInfo,
			       bool createMatrixData,
			       bool storeMatrixData)
  {
    FUNCNAME("PetscProblemStat::solve()");

    TEST_EXIT(meshDistributor)("Should not happen!\n");

    double wtime = MPI::Wtime();

    if (createMatrixData) {
      petscSolver->setMeshDistributor(meshDistributor);
      petscSolver->fillPetscMatrix(systemMatrix);
    }

    petscSolver->fillPetscRhs(rhs);

    petscSolver->solvePetscMatrix(*solution, adaptInfo);   

    petscSolver->destroyVectorData();

    if (!storeMatrixData)
      petscSolver->destroyMatrixData();

    INFO(info, 8)("solution of discrete system needed %.5f seconds\n", 
		  MPI::Wtime() - wtime);
  }

}
